@@ -22,6 +22,7 @@
  ******************************************************************************/
 #include "packagecheck.h"
 
+#include "../../geometry/polygon.h"
 #include "../../utils/toolbox.h"
 #include "../../utils/transform.h"
 #include "package.h"
@@ -58,6 +59,7 @@ RuleCheckMessageList PackageCheck::runChecks() const {
   checkWrongTextLayers(msgs);
   checkPackageOutlines(msgs);
   checkCourtyards(msgs);
+  checkOriginInCenter(msgs);
   checkPadsPackagePadUuid(msgs);
   checkPadsClearanceToPads(msgs);
   checkPadsClearanceToLegend(msgs);
@@ -69,6 +71,7 @@ RuleCheckMessageList PackageCheck::runChecks() const {
   checkCopperClearanceOnPads(msgs);
   checkPadFunctions(msgs);
   checkHolesStopMask(msgs);
+  checkLineWidths(msgs);
   checkZones(msgs);
   checkFootprintModels(msgs);
   return msgs;
@@ -193,6 +196,113 @@ void PackageCheck::checkCourtyards(MsgList& msgs) const {
   }
 }
 
+void PackageCheck::checkOriginInCenter(MsgList& msgs) const {
+  for (auto itFtp = mPackage.getFootprints().begin();
+       itFtp != mPackage.getFootprints().end(); ++itFtp) {
+    // Specify a relative and an absolute tolerance.
+    qreal toleranceFactor = 0.01;
+    Length toleranceAbs(100000);
+
+    // Allow higher tolerance for THT parts.
+    if (mPackage.getAssemblyType(false) != Package::AssemblyType::Smt) {
+      toleranceFactor = 0.05;
+      toleranceAbs = Length(635000);
+    }
+
+    // Helper to increase tolerance.
+    auto increaseToleranceFactor = [&](qreal newFactor) {
+      toleranceFactor = std::max(newFactor, toleranceFactor);
+    };
+
+    // Consider mainly the package outlines since this is the most reliable
+    // source for this check.
+    QSet<Length> x, y;
+    for (const Circle& c : (*itFtp).getCircles()) {
+      if (c.getLayer() == Layer::topPackageOutlines()) {
+        const Length r = c.getDiameter() / 2;
+        x.insert(c.getCenter().getX() - r);
+        x.insert(c.getCenter().getX() + r);
+        y.insert(c.getCenter().getY() - r);
+        y.insert(c.getCenter().getY() + r);
+      }
+    }
+    for (const Polygon& p : (*itFtp).getPolygons()) {
+      if (p.getLayer() == Layer::topPackageOutlines()) {
+        if (p.getPath().isCurved()) {
+          const QRectF rectPx =
+              p.getPath().toClosedPath().toQPainterPathPx().boundingRect();
+          const Point topLeft = Point::fromPx(rectPx.topLeft());
+          const Point botRight = Point::fromPx(rectPx.bottomRight());
+          x.insert(topLeft.getX());
+          x.insert(botRight.getX());
+          y.insert(topLeft.getY());
+          y.insert(botRight.getY());
+          increaseToleranceFactor(0.15);  // Relax check due to complex shape.
+        } else {
+          for (const Vertex& v : p.getPath().getVertices()) {
+            x.insert(v.getPos().getX());
+            y.insert(v.getPos().getY());
+          }
+          if (p.getPath().toOpenPath().cleaned().getVertices().count() != 4) {
+            increaseToleranceFactor(0.1);  // Relax check due to complex shape.
+          }
+        }
+      }
+    }
+
+    // Only if we didn't find a package body, take more objects into account
+    // but allow much larger tolerance.
+    if (x.isEmpty() || y.isEmpty()) {
+      increaseToleranceFactor(0.3);
+      for (const FootprintPad& pad : (*itFtp).getPads()) {
+        x.insert(pad.getPosition().getX());
+        y.insert(pad.getPosition().getY());
+      }
+      for (const Circle& c : (*itFtp).getCircles()) {
+        if (c.getLayer() == Layer::topDocumentation()) {
+          const Length r = c.getDiameter() / 2;
+          x.insert(c.getCenter().getX() - r);
+          x.insert(c.getCenter().getX() + r);
+          y.insert(c.getCenter().getY() - r);
+          y.insert(c.getCenter().getY() + r);
+        }
+      }
+      for (const Polygon& p : (*itFtp).getPolygons()) {
+        if (p.getLayer() == Layer::topDocumentation()) {
+          const QRectF rectPx =
+              p.getPath().toClosedPath().toQPainterPathPx().boundingRect();
+          const Point topLeft = Point::fromPx(rectPx.topLeft());
+          const Point botRight = Point::fromPx(rectPx.bottomRight());
+          x.insert(topLeft.getX());
+          x.insert(botRight.getX());
+          y.insert(topLeft.getY());
+          y.insert(botRight.getY());
+        }
+      }
+    }
+
+    // If there is no boundary, abort.
+    if (x.isEmpty() || y.isEmpty()) {
+      continue;
+    }
+
+    // Calculate and check center.
+    const Length minX = *std::min_element(x.begin(), x.end());
+    const Length maxX = *std::max_element(x.begin(), x.end());
+    const Length minY = *std::min_element(y.begin(), y.end());
+    const Length maxY = *std::max_element(y.begin(), y.end());
+    const Point center((minX + maxX) / 2, (minY + maxY) / 2);
+    const Length width = maxX - minX;
+    const Length height = maxY - minY;
+    const Length tolX = std::max(width.scaled(toleranceFactor), toleranceAbs);
+    const Length tolY = std::max(height.scaled(toleranceFactor), toleranceAbs);
+    if ((center.getX().abs() > tolX) || (center.getY().abs() > tolY)) {
+      msgs.append(
+          std::make_shared<MsgFootprintOriginNotInCenter>(itFtp.ptr(), center));
+    }
+  }
+}
+
 void PackageCheck::checkPadsPackagePadUuid(MsgList& msgs) const {
   for (auto itFtp = mPackage.getFootprints().begin();
        itFtp != mPackage.getFootprints().end(); ++itFtp) {
@@ -575,6 +685,48 @@ void PackageCheck::checkHolesStopMask(MsgList& msgs) const {
   }
 }
 
+void PackageCheck::checkLineWidths(MsgList& msgs) const {
+  const QList<const Layer*> silkscreenLayers = {
+      &Layer::topLegend(), &Layer::botLegend(), &Layer::topNames(),
+      &Layer::botNames(),  &Layer::topValues(), &Layer::botValues(),
+  };
+  const Length minSilkscreenWidth(150000);  // See BoardDesignRuleCheckSettings.
+
+  for (auto itFtp = mPackage.getFootprints().begin();
+       itFtp != mPackage.getFootprints().end(); ++itFtp) {
+    std::shared_ptr<const Footprint> footprint = itFtp.ptr();
+    for (auto itPolygon = (*itFtp).getPolygons().begin();
+         itPolygon != (*itFtp).getPolygons().end(); ++itPolygon) {
+      std::shared_ptr<const Polygon> polygon = itPolygon.ptr();
+      if (silkscreenLayers.contains(&polygon->getLayer()) &&
+          (polygon->getLineWidth() < minSilkscreenWidth) &&
+          (!polygon->isFilled())) {
+        msgs.append(std::make_shared<MsgMinimumWidthViolation>(
+            footprint, polygon, minSilkscreenWidth));
+      }
+    }
+    for (auto itCircle = (*itFtp).getCircles().begin();
+         itCircle != (*itFtp).getCircles().end(); ++itCircle) {
+      std::shared_ptr<const Circle> circle = itCircle.ptr();
+      if (silkscreenLayers.contains(&circle->getLayer()) &&
+          (circle->getLineWidth() < minSilkscreenWidth) &&
+          (!circle->isFilled())) {
+        msgs.append(std::make_shared<MsgMinimumWidthViolation>(
+            footprint, circle, minSilkscreenWidth));
+      }
+    }
+    for (auto itText = (*itFtp).getStrokeTexts().begin();
+         itText != (*itFtp).getStrokeTexts().end(); ++itText) {
+      std::shared_ptr<const StrokeText> text = itText.ptr();
+      if (silkscreenLayers.contains(&text->getLayer()) &&
+          (text->getStrokeWidth() < minSilkscreenWidth)) {
+        msgs.append(std::make_shared<MsgMinimumWidthViolation>(
+            footprint, text, minSilkscreenWidth));
+      }
+    }
+  }
+}
+
 void PackageCheck::checkZones(MsgList& msgs) const {
   for (auto itFtp = mPackage.getFootprints().begin();
        itFtp != mPackage.getFootprints().end(); ++itFtp) {
