@@ -23,8 +23,11 @@
 #include "packageeditorwidget.h"
 
 #include "../../3d/openglscenebuilder.h"
+#include "../../cmd/cmdcircleedit.h"
 #include "../../cmd/cmdholeedit.h"
+#include "../../cmd/cmdpolygonedit.h"
 #include "../../cmd/cmdstroketextedit.h"
+#include "../../dialogs/backgroundimagesetupdialog.h"
 #include "../../dialogs/gridsettingsdialog.h"
 #include "../../editorcommandset.h"
 #include "../../graphics/graphicsscene.h"
@@ -32,6 +35,7 @@
 #include "../../utils/toolbarproxy.h"
 #include "../../widgets/openglview.h"
 #include "../../widgets/statusbar.h"
+#include "../../widgets/unsignedlengthedit.h"
 #include "../../workspace/desktopservices.h"
 #include "../cmd/cmdfootprintedit.h"
 #include "../cmd/cmdfootprintpadedit.h"
@@ -40,12 +44,14 @@
 #include "ui_packageeditorwidget.h"
 
 #include <librepcb/core/application.h>
+#include <librepcb/core/fileio/fileutils.h>
 #include <librepcb/core/font/stroketextpathbuilder.h>
 #include <librepcb/core/library/librarybaseelementcheckmessages.h>
 #include <librepcb/core/library/libraryelementcheckmessages.h>
 #include <librepcb/core/library/pkg/footprintpainter.h>
 #include <librepcb/core/library/pkg/package.h>
 #include <librepcb/core/library/pkg/packagecheckmessages.h>
+#include <librepcb/core/serialization/sexpression.h>
 #include <librepcb/core/types/pcbcolor.h>
 #include <librepcb/core/utils/clipperhelpers.h>
 #include <librepcb/core/workspace/workspace.h>
@@ -58,8 +64,112 @@
  *  Namespace
  ******************************************************************************/
 namespace librepcb {
+
+template <>
+std::unique_ptr<SExpression> serialize(const float& obj) {
+  QString s = QString::number(obj, 'f', 6);
+  while (s.endsWith("0") && (!s.endsWith(".0"))) {
+    s.chop(1);
+  }
+  return SExpression::createToken(s);
+}
+
+template <>
+std::unique_ptr<SExpression> serialize(const double& obj) {
+  QString s = QString::number(obj, 'f', 6);
+  while (s.endsWith("0") && (!s.endsWith(".0"))) {
+    s.chop(1);
+  }
+  return SExpression::createToken(s);
+}
+
 namespace editor {
 
+/*******************************************************************************
+ *  Class BackgroundImageSettings
+ ******************************************************************************/
+
+bool BackgroundImageSettings::tryLoadFromDir(const FilePath& dir) noexcept {
+  *this = BackgroundImageSettings();  // Reset.
+
+  try {
+    const FilePath fp = dir.getPathTo("settings.lp");
+    if (fp.isExistingFile()) {
+      image.load(dir.getPathTo("image.png").toStr(), "png");
+      std::unique_ptr<SExpression> root =
+          SExpression::parse(FileUtils::readFile(fp), fp);
+      enabled = deserialize<bool>(root->getChild("enabled/@0"));
+      rotation = deserialize<Angle>(root->getChild("rotation/@0"));
+      foreach (const SExpression* node, root->getChildren("reference")) {
+        const QPointF source(deserialize<qreal>(node->getChild("source/@0")),
+                             deserialize<qreal>(node->getChild("source/@1")));
+        const Point target(node->getChild("target"));
+        references.append(std::make_pair(source, target));
+      }
+      return true;
+    }
+  } catch (const Exception& e) {
+    qWarning() << "Failed to load background image data:" << e.getMsg();
+  }
+  return false;
+}
+
+void BackgroundImageSettings::saveToDir(const FilePath& dir) noexcept {
+  try {
+    if (!image.isNull()) {
+      FileUtils::makePath(dir);
+      image.save(dir.getPathTo("image.png").toStr(), "png");
+      std::unique_ptr<SExpression> root =
+          SExpression::createList("librepcb_background_image");
+      root->ensureLineBreak();
+      root->appendChild("enabled", enabled);
+      root->ensureLineBreak();
+      root->appendChild("rotation", rotation);
+      for (const auto& ref : references) {
+        root->ensureLineBreak();
+        SExpression& refNode = root->appendList("reference");
+        SExpression& sourceNode = refNode.appendList("source");
+        sourceNode.appendChild(ref.first.x());
+        sourceNode.appendChild(ref.first.y());
+        ref.second.serialize(refNode.appendList("target"));
+      }
+      root->ensureLineBreak();
+      FileUtils::writeFile(dir.getPathTo("settings.lp"), root->toByteArray());
+    } else if (dir.isExistingDir()) {
+      FileUtils::removeDirRecursively(dir);
+    }
+  } catch (const Exception& e) {
+    qWarning() << "Failed to save background image data:" << e.getMsg();
+  }
+}
+
+QPixmap BackgroundImageSettings::buildPixmap(
+    const QColor& bgColor) const noexcept {
+  QImage img = image.convertToFormat(QImage::Format_ARGB32);
+
+  auto colorDiff = [](const QColor& a, const QColor& b) {
+    return std::abs(a.lightnessF() - b.lightnessF());
+  };
+
+  // If the image background color is the inverse of the graphics view
+  // background, invert the image to get good contrast for lines in the image.
+  if (colorDiff(img.pixelColor(0, 0), bgColor) > 0.5) {
+    img.invertPixels();
+  }
+
+  // Make the image background transparent.
+  const QColor imgBgColor = img.pixelColor(0, 0);
+  for (int i = 0; i < img.width(); ++i) {
+    for (int k = 0; k < img.height(); ++k) {
+      if (colorDiff(img.pixelColor(i, k), imgBgColor) < 0.3) {
+        img.setPixelColor(i, k, Qt::transparent);
+      }
+    }
+  }
+
+  return QPixmap::fromImage(img);
+}
+
 /*******************************************************************************
  *  Constructors / Destructor
  ******************************************************************************/
@@ -69,7 +179,8 @@ PackageEditorWidget::PackageEditorWidget(const Context& context,
   : EditorWidgetBase(context, fp, parent),
     mUi(new Ui::PackageEditorWidget),
     mGraphicsScene(new GraphicsScene()),
-    mOpenGlSceneBuildScheduled(false) {
+    mOpenGlSceneBuildScheduled(false),
+    mBackgroundImageGraphicsItem(new QGraphicsPixmapItem()) {
   mUi->setupUi(this);
   mUi->lstMessages->setHandler(this);
   mUi->lstMessages->setReadOnly(mContext.readOnly);
@@ -230,6 +341,16 @@ PackageEditorWidget::PackageEditorWidget(const Context& context,
           &PackageEditorWidget::setStatusBarMessage);
   currentFootprintChanged(0);  // small hack to select the first footprint...
 
+  // Setup background image.
+  mBackgroundImageGraphicsItem->setFlag(QGraphicsItem::ItemIsSelectable, true);
+  mBackgroundImageGraphicsItem->setTransformationMode(Qt::SmoothTransformation);
+  mBackgroundImageGraphicsItem->setZValue(-1000);
+  mBackgroundImageGraphicsItem->setOpacity(0.8);
+  mBackgroundImageGraphicsItem->setVisible(false);
+  mGraphicsScene->addItem(*mBackgroundImageGraphicsItem);
+  mBackgroundImageSettings.tryLoadFromDir(getBackgroundImageCacheDir());
+  applyBackgroundImageSettings();
+
   // Last but not least, connect the graphics scene events with the FSM.
   mUi->graphicsView->setEventHandlerObject(this);
 }
@@ -254,15 +375,21 @@ PackageEditorWidget::~PackageEditorWidget() noexcept {
  *  Getters
  ******************************************************************************/
 
+bool PackageEditorWidget::isBackgroundImageSet() const noexcept {
+  return mBackgroundImageGraphicsItem->isVisible();
+}
+
 QSet<EditorWidgetBase::Feature> PackageEditorWidget::getAvailableFeatures()
     const noexcept {
   QSet<EditorWidgetBase::Feature> features = {
       EditorWidgetBase::Feature::Close,
       EditorWidgetBase::Feature::GraphicsView,
       EditorWidgetBase::Feature::OpenGlView,
+      EditorWidgetBase::Feature::BackgroundImage,
       EditorWidgetBase::Feature::ExportGraphics,
       EditorWidgetBase::Feature::GenerateOutline,
       EditorWidgetBase::Feature::GenerateCourtyard,
+      EditorWidgetBase::Feature::ReNumberPads,
   };
   return features + mFsm->getAvailableFeatures();
 }
@@ -293,6 +420,7 @@ void PackageEditorWidget::connectEditor(
   mToolsActionGroup->setActionEnabled(Tool::DRAW_ZONE, enabled);
   mToolsActionGroup->setActionEnabled(Tool::ADD_HOLES, enabled);
   mToolsActionGroup->setActionEnabled(Tool::MEASURE, true);
+  mToolsActionGroup->setActionEnabled(Tool::RENUMBER_PADS, enabled);
   mToolsActionGroup->setCurrentAction(mFsm->getCurrentTool());
   connect(mFsm.data(), &PackageEditorFsm::toolChanged, mToolsActionGroup,
           &ExclusiveActionGroup::setCurrentAction);
@@ -359,7 +487,31 @@ bool PackageEditorWidget::paste() noexcept {
 }
 
 bool PackageEditorWidget::move(Qt::ArrowType direction) noexcept {
-  return mFsm->processMove(direction);
+  Point delta;
+  switch (direction) {
+    case Qt::LeftArrow: {
+      delta.setX(-mUi->graphicsView->getGridInterval());
+      break;
+    }
+    case Qt::RightArrow: {
+      delta.setX(*mUi->graphicsView->getGridInterval());
+      break;
+    }
+    case Qt::UpArrow: {
+      delta.setY(*mUi->graphicsView->getGridInterval());
+      break;
+    }
+    case Qt::DownArrow: {
+      delta.setY(-mUi->graphicsView->getGridInterval());
+      break;
+    }
+    default: {
+      qWarning() << "Unhandled switch-case in PackageEditorWidget::move():"
+                 << direction;
+      break;
+    }
+  }
+  return mFsm->processMove(delta);
 }
 
 bool PackageEditorWidget::rotate(const Angle& rotation) noexcept {
@@ -374,6 +526,10 @@ bool PackageEditorWidget::flip(Qt::Orientation orientation) noexcept {
   return mFsm->processFlip(orientation);
 }
 
+bool PackageEditorWidget::moveAlign() noexcept {
+  return mFsm->processMoveAlign();
+}
+
 bool PackageEditorWidget::snapToGrid() noexcept {
   return mFsm->processSnapToGrid();
 }
@@ -459,6 +615,36 @@ bool PackageEditorWidget::decreaseGridInterval() noexcept {
   return true;
 }
 
+bool PackageEditorWidget::toggleBackgroundImage() noexcept {
+  if (mBackgroundImageGraphicsItem->isVisible()) {
+    mBackgroundImageSettings.enabled = false;
+  } else {
+    // Show dialog.
+    BackgroundImageSetupDialog dlg("package_editor", this);
+    if (!mBackgroundImageSettings.image.isNull()) {
+      dlg.setData(mBackgroundImageSettings.image,
+                  mBackgroundImageSettings.rotation,
+                  mBackgroundImageSettings.references);
+    }
+    if (dlg.exec() != QDialog::Accepted) {
+      return mBackgroundImageGraphicsItem->isVisible();  // Aborted.
+    }
+
+    mBackgroundImageSettings.image = dlg.getImage();
+    mBackgroundImageSettings.rotation = dlg.getRotation();
+    mBackgroundImageSettings.references = dlg.getReferences();
+    mBackgroundImageSettings.enabled =
+        (!mBackgroundImageSettings.image.isNull()) &&
+        (mBackgroundImageSettings.references.count() >= 2);
+    toggle3DMode(false);
+  }
+
+  // Store & apply new settings.
+  mBackgroundImageSettings.saveToDir(getBackgroundImageCacheDir());
+  applyBackgroundImageSettings();
+  return mBackgroundImageGraphicsItem->isVisible();
+}
+
 /*******************************************************************************
  *  Private Methods
  ******************************************************************************/
@@ -604,6 +790,8 @@ bool PackageEditorWidget::toolChangeRequested(Tool newTool,
       return mFsm->processStartAddingHoles();
     case Tool::MEASURE:
       return mFsm->processStartMeasure();
+    case Tool::RENUMBER_PADS:
+      return mFsm->processStartReNumberPads();
     default:
       return false;
   }
@@ -769,6 +957,48 @@ void PackageEditorWidget::fixMsg(const MsgMissingPackageOutline& msg) {
   mFsm->processGenerateOutline();
 }
 
+template <>
+void PackageEditorWidget::fixMsg(const MsgMinimumWidthViolation& msg) {
+  if (!mCurrentFootprint) return;
+
+  QDialog dlg(this);
+  dlg.setWindowTitle(tr("New Line Width"));
+  QVBoxLayout* vLayout = new QVBoxLayout(&dlg);
+  UnsignedLengthEdit* edtWidth = new UnsignedLengthEdit(&dlg);
+  edtWidth->configure(mLengthUnit, LengthEditBase::Steps::generic(),
+                      "package_editor/fix_minimum_width_dialog");
+  edtWidth->setValue(UnsignedLength(200000));
+  edtWidth->setFocus();
+  vLayout->addWidget(edtWidth);
+  QDialogButtonBox* btnBox = new QDialogButtonBox(&dlg);
+  btnBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+  connect(btnBox, &QDialogButtonBox::rejected, &dlg, &QDialog::reject);
+  connect(btnBox, &QDialogButtonBox::accepted, &dlg, &QDialog::accept);
+  vLayout->addWidget(btnBox);
+  if (dlg.exec() != QDialog::Accepted) {
+    return;
+  }
+
+  if (auto p = mCurrentFootprint->getPolygons().find(msg.getPolygon().get())) {
+    QScopedPointer<CmdPolygonEdit> cmd(new CmdPolygonEdit(*p));
+    cmd->setLineWidth(edtWidth->getValue(), false);
+    mUndoStack->execCmd(cmd.take());
+  } else if (auto c =
+                 mCurrentFootprint->getCircles().find(msg.getCircle().get())) {
+    QScopedPointer<CmdCircleEdit> cmd(new CmdCircleEdit(*c));
+    cmd->setLineWidth(edtWidth->getValue(), false);
+    mUndoStack->execCmd(cmd.take());
+  } else if (auto t = mCurrentFootprint->getStrokeTexts().find(
+                 msg.getStrokeText().get())) {
+    QScopedPointer<CmdStrokeTextEdit> cmd(new CmdStrokeTextEdit(*t));
+    cmd->setStrokeWidth(edtWidth->getValue(), false);
+    mUndoStack->execCmd(cmd.take());
+  } else {
+    throw LogicError(__FILE__, __LINE__,
+                     "Whoops, not implemented! Please open a bug report.");
+  }
+}
+
 template <>
 void PackageEditorWidget::fixMsg(const MsgMissingCourtyard& msg) {
   mUi->footprintEditorWidget->setCurrentIndex(
@@ -806,6 +1036,17 @@ void PackageEditorWidget::fixMsg(const MsgMissingFootprintValue& msg) {
   mFsm->processStartAddingValues();
 }
 
+template <>
+void PackageEditorWidget::fixMsg(const MsgFootprintOriginNotInCenter& msg) {
+  mFsm->processAbortCommand();
+  mFsm->processAbortCommand();
+  currentFootprintChanged(
+      mPackage->getFootprints().indexOf(msg.getFootprint().get()));
+  mFsm->processSelectAll();
+  mFsm->processMove(-msg.getCenter());
+  mFsm->processAbortCommand();  // Clear selection.
+}
+
 template <>
 void PackageEditorWidget::fixMsg(const MsgWrongFootprintTextLayer& msg) {
   std::shared_ptr<Footprint> footprint =
@@ -988,11 +1229,13 @@ bool PackageEditorWidget::processRuleCheckMessage(
   if (fixMsgHelper<MsgMissingAuthor>(msg, applyFix)) return true;
   if (fixMsgHelper<MsgMissingCategories>(msg, applyFix)) return true;
   if (fixMsgHelper<MsgMissingPackageOutline>(msg, applyFix)) return true;
+  if (fixMsgHelper<MsgMinimumWidthViolation>(msg, applyFix)) return true;
   if (fixMsgHelper<MsgMissingCourtyard>(msg, applyFix)) return true;
   if (fixMsgHelper<MsgMissingFootprint>(msg, applyFix)) return true;
   if (fixMsgHelper<MsgMissingFootprintModel>(msg, applyFix)) return true;
   if (fixMsgHelper<MsgMissingFootprintName>(msg, applyFix)) return true;
   if (fixMsgHelper<MsgMissingFootprintValue>(msg, applyFix)) return true;
+  if (fixMsgHelper<MsgFootprintOriginNotInCenter>(msg, applyFix)) return true;
   if (fixMsgHelper<MsgWrongFootprintTextLayer>(msg, applyFix)) return true;
   if (fixMsgHelper<MsgUnusedCustomPadOutline>(msg, applyFix)) return true;
   if (fixMsgHelper<MsgInvalidCustomPadOutline>(msg, applyFix)) return true;
@@ -1069,6 +1312,51 @@ void PackageEditorWidget::setGridProperties(const PositiveLength& interval,
   }
 }
 
+void PackageEditorWidget::applyBackgroundImageSettings() noexcept {
+  BackgroundImageSettings& s = mBackgroundImageSettings;
+
+  const bool enable = s.enabled && (!s.image.isNull());
+  mBackgroundImageGraphicsItem->setVisible(enable);
+
+  if (enable) {
+    // Make the image background transparent.
+    const Theme& theme = mContext.workspace.getSettings().themes.getActive();
+    mBackgroundImageGraphicsItem->setPixmap(s.buildPixmap(
+        theme.getColor(Theme::Color::sBoardBackground).getPrimaryColor()));
+
+    // Apply the transform.
+    QTransform t;
+    t.rotate(-s.rotation.toDeg());
+    if (s.references.count() >= 2) {
+      const Point deltaPx =
+          Point::fromPx(s.references.at(1).first - s.references.at(0).first)
+              .rotated(-s.rotation);
+      const Point deltaMm =
+          (s.references.at(1).second - s.references.at(0).second);
+
+      const qreal scaleFactorX =
+          std::abs(deltaMm.toMmQPointF().x() / deltaPx.toMmQPointF().x());
+      const qreal scaleFactorY =
+          std::abs(deltaMm.toMmQPointF().y() / deltaPx.toMmQPointF().y());
+
+      t.scale(scaleFactorX, scaleFactorY);
+      t.translate(-s.references.first().first.x(),
+                  -s.references.first().first.y());
+    }
+    mBackgroundImageGraphicsItem->setTransform(t);
+    if (s.references.count() >= 1) {
+      mBackgroundImageGraphicsItem->setPos(
+          s.references.first().second.toPxQPointF());
+    }
+  }
+}
+
+FilePath PackageEditorWidget::getBackgroundImageCacheDir() const noexcept {
+  return Application::getCacheDir()
+      .getPathTo("backgrounds")
+      .getPathTo(mPackage->getUuid().toStr());
+}
+
 void PackageEditorWidget::toggle3DMode(bool enable) noexcept {
   if (enable) {
     mUi->graphicsView->hide();
