@@ -371,14 +371,16 @@ bool SymbolEditorState_Select::processCopy() noexcept {
   }
 }
 
-bool SymbolEditorState_Select::processPaste() noexcept {
+bool SymbolEditorState_Select::processPaste(
+    std::unique_ptr<SymbolClipboardData> data) noexcept {
   switch (mState) {
     case SubState::IDLE: {
       try {
         // Get footprint items from clipboard, if none provided.
-        std::unique_ptr<SymbolClipboardData> data =
-            SymbolClipboardData::fromMimeData(
-                qApp->clipboard()->mimeData());  // can throw
+        if (!data) {
+          data = SymbolClipboardData::fromMimeData(
+              qApp->clipboard()->mimeData());  // can throw
+        }
         if (data) {
           return startPaste(std::move(data), tl::nullopt);
         }
@@ -396,35 +398,10 @@ bool SymbolEditorState_Select::processPaste() noexcept {
   return false;
 }
 
-bool SymbolEditorState_Select::processMove(Qt::ArrowType direction) noexcept {
+bool SymbolEditorState_Select::processMove(const Point& delta) noexcept {
   switch (mState) {
     case SubState::IDLE: {
       try {
-        Point delta;
-        switch (direction) {
-          case Qt::LeftArrow: {
-            delta.setX(-getGridInterval());
-            break;
-          }
-          case Qt::RightArrow: {
-            delta.setX(*getGridInterval());
-            break;
-          }
-          case Qt::UpArrow: {
-            delta.setY(*getGridInterval());
-            break;
-          }
-          case Qt::DownArrow: {
-            delta.setY(-getGridInterval());
-            break;
-          }
-          default: {
-            qWarning() << "Unhandled switch-case in "
-                          "SymbolEditorState_Select::processMove():"
-                       << direction;
-            break;
-          }
-        }
         QScopedPointer<CmdDragSelectedSymbolItems> cmd(
             new CmdDragSelectedSymbolItems(mContext));
         cmd->translate(delta);
@@ -658,7 +635,7 @@ bool SymbolEditorState_Select::openContextMenuAtPos(const Point& pos) noexcept {
 
   // If a polygon line is under the cursor, add vertex menu items
   if (auto i = std::dynamic_pointer_cast<PolygonGraphicsItem>(selectedItem)) {
-    if (auto polygon = mContext.symbol.getPolygons().find(&i->getPolygon())) {
+    if (auto polygon = mContext.symbol.getPolygons().find(&i->getObj())) {
       QVector<int> vertices = i->getVertexIndicesAtPosition(pos);
       if (!vertices.isEmpty()) {
         QAction* aRemoveVertex = cmd.vertexRemove.createAction(
@@ -738,13 +715,13 @@ bool SymbolEditorState_Select::openPropertiesDialogOfItem(
 
   if (auto i = std::dynamic_pointer_cast<SymbolPinGraphicsItem>(item)) {
     SymbolPinPropertiesDialog dialog(
-        i->getPin(), mContext.undoStack, getLengthUnit(),
+        i->getPtr(), mContext.undoStack, getLengthUnit(),
         "symbol_editor/pin_properties_dialog", &mContext.editorWidget);
     dialog.setReadOnly(mContext.editorContext.readOnly);
     dialog.exec();
     return true;
   } else if (auto i = std::dynamic_pointer_cast<TextGraphicsItem>(item)) {
-    TextPropertiesDialog dialog(i->getText(), mContext.undoStack,
+    TextPropertiesDialog dialog(i->getObj(), mContext.undoStack,
                                 getAllowedTextLayers(), getLengthUnit(),
                                 "symbol_editor/text_properties_dialog",
                                 &mContext.editorWidget);
@@ -753,7 +730,7 @@ bool SymbolEditorState_Select::openPropertiesDialogOfItem(
     return true;
   } else if (auto i = std::dynamic_pointer_cast<PolygonGraphicsItem>(item)) {
     PolygonPropertiesDialog dialog(
-        i->getPolygon(), mContext.undoStack, getAllowedCircleAndPolygonLayers(),
+        i->getObj(), mContext.undoStack, getAllowedCircleAndPolygonLayers(),
         getLengthUnit(), "symbol_editor/polygon_properties_dialog",
         &mContext.editorWidget);
     dialog.setReadOnly(mContext.editorContext.readOnly);
@@ -761,7 +738,7 @@ bool SymbolEditorState_Select::openPropertiesDialogOfItem(
     return true;
   } else if (auto i = std::dynamic_pointer_cast<CircleGraphicsItem>(item)) {
     CirclePropertiesDialog dialog(
-        i->getCircle(), mContext.undoStack, getAllowedCircleAndPolygonLayers(),
+        i->getObj(), mContext.undoStack, getAllowedCircleAndPolygonLayers(),
         getLengthUnit(), "symbol_editor/circle_properties_dialog",
         &mContext.editorWidget);
     dialog.setReadOnly(mContext.editorContext.readOnly);
@@ -790,23 +767,22 @@ bool SymbolEditorState_Select::copySelectedItemsToClipboard() noexcept {
     foreach (const std::shared_ptr<SymbolPinGraphicsItem>& pin,
              mContext.symbolGraphicsItem.getSelectedPins()) {
       Q_ASSERT(pin);
-      data.getPins().append(std::make_shared<SymbolPin>(*pin->getPin()));
+      data.getPins().append(std::make_shared<SymbolPin>(pin->getObj()));
     }
     foreach (const std::shared_ptr<CircleGraphicsItem>& circle,
              mContext.symbolGraphicsItem.getSelectedCircles()) {
       Q_ASSERT(circle);
-      data.getCircles().append(std::make_shared<Circle>(circle->getCircle()));
+      data.getCircles().append(std::make_shared<Circle>(circle->getObj()));
     }
     foreach (const std::shared_ptr<PolygonGraphicsItem>& polygon,
              mContext.symbolGraphicsItem.getSelectedPolygons()) {
       Q_ASSERT(polygon);
-      data.getPolygons().append(
-          std::make_shared<Polygon>(polygon->getPolygon()));
+      data.getPolygons().append(std::make_shared<Polygon>(polygon->getObj()));
     }
     foreach (const std::shared_ptr<TextGraphicsItem>& text,
              mContext.symbolGraphicsItem.getSelectedTexts()) {
       Q_ASSERT(text);
-      data.getTexts().append(std::make_shared<Text>(text->getText()));
+      data.getTexts().append(std::make_shared<Text>(text->getObj()));
     }
     if (data.getItemCount() > 0) {
       qApp->clipboard()->setMimeData(
