@@ -31,6 +31,7 @@
 #include "../../project/cmd/cmdboardadd.h"
 #include "../../project/cmd/cmdboardremove.h"
 #include "../../undostack.h"
+#include "../../utils/editortoolbox.h"
 #include "../../utils/exclusiveactiongroup.h"
 #include "../../utils/menubuilder.h"
 #include "../../utils/standardeditorcommandhandler.h"
@@ -42,6 +43,7 @@
 #include "../../widgets/searchtoolbar.h"
 #include "../../workspace/desktopservices.h"
 #include "../bomgeneratordialog.h"
+#include "../cmd/cmdboardspecctraimport.h"
 #include "../outputjobsdialog/outputjobsdialog.h"
 #include "../projecteditor.h"
 #include "../projectsetupdialog.h"
@@ -65,6 +67,7 @@
 #include <librepcb/core/project/board/boardd356netlistexport.h>
 #include <librepcb/core/project/board/boardpainter.h>
 #include <librepcb/core/project/board/boardplanefragmentsbuilder.h>
+#include <librepcb/core/project/board/boardspecctraexport.h>
 #include <librepcb/core/project/board/drc/boarddesignrulecheck.h>
 #include <librepcb/core/project/board/items/bi_device.h>
 #include <librepcb/core/project/board/items/bi_footprintpad.h>
@@ -79,6 +82,7 @@
 #include <librepcb/core/project/project.h>
 #include <librepcb/core/project/projectattributelookup.h>
 #include <librepcb/core/types/layer.h>
+#include <librepcb/core/utils/messagelogger.h>
 #include <librepcb/core/utils/scopeguard.h>
 #include <librepcb/core/utils/toolbox.h>
 #include <librepcb/core/workspace/workspace.h>
@@ -115,12 +119,21 @@ BoardEditor::BoardEditor(ProjectEditor& projectEditor, Project& project)
     mTimestampOfLastOpenGlSceneRebuild(0),
     mVisibleSceneRect(),
     mFsm(),
-    mPlaneFragmentsBuilder(new BoardPlaneFragmentsBuilder(true, this)),
+    mPlaneFragmentsBuilder(new BoardPlaneFragmentsBuilder(this)),
     mTimestampOfLastPlaneRebuild(0) {
   mUi->setupUi(this);
   mUi->tabBar->setDocumentMode(true);  // For MacOS
   mUi->lblUnplacedComponentsNote->hide();
 
+#if QT_VERSION_MAJOR >= 6
+  // Workaround for automatically closing window when opening 3D viewer,
+  // see https://github.com/LibrePCB/LibrePCB/issues/1363.
+  {
+    QOpenGLWidget* w = new QOpenGLWidget(this);
+    w->hide();
+  }
+#endif
+
   // Setup graphics view.
   const Theme& theme =
       mProjectEditor.getWorkspace().getSettings().themes.getActive();
@@ -151,9 +164,6 @@ BoardEditor::BoardEditor(ProjectEditor& projectEditor, Project& project)
           &BoardEditor::hide3DView);
   connect(&mProjectEditor.getUndoStack(), &UndoStack::stateModified, this,
           &BoardEditor::scheduleOpenGlSceneUpdate);
-  connect(mPlaneFragmentsBuilder.data(),
-          &BoardPlaneFragmentsBuilder::boardPlanesModified, this,
-          &BoardEditor::scheduleOpenGlSceneUpdate);
 
   // Setup status bar.
   mUi->statusbar->setFields(StatusBar::AbsolutePosition |
@@ -197,7 +207,12 @@ BoardEditor::BoardEditor(ProjectEditor& projectEditor, Project& project)
 
   // Setup plane rebuilder.
   connect(mPlaneFragmentsBuilder.data(), &BoardPlaneFragmentsBuilder::finished,
-          this, [this]() {
+          this, [this](BoardPlaneFragmentsBuilder::Result result) {
+            if (result.applyToBoard() && result.board) {
+              // Board has been modified, update air wires & 3D view.
+              result.board->forceAirWiresRebuild();
+              scheduleOpenGlSceneUpdate();
+            }
             mTimestampOfLastPlaneRebuild = QDateTime::currentMSecsSinceEpoch();
           });
 
@@ -304,6 +319,7 @@ bool BoardEditor::setActiveBoardIndex(int index) noexcept {
       storeLayersVisibility();
     }
 
+    clearDrcMarker();  // Avoid dangling pointers.
     mUi->graphicsView->setScene(nullptr);
     mGraphicsScene.reset();
     mActiveBoard = newBoard;
@@ -584,6 +600,8 @@ void BoardEditor::createActions() noexcept {
       this, this, [this]() { runDrc(false); }));
   mActionImportDxf.reset(cmd.importDxf.createAction(
       this, mFsm.data(), &BoardEditorFsm::processImportDxf));
+  mActionImportSpecctra.reset(cmd.importSpecctraSes.createAction(
+      this, this, &BoardEditor::execSpecctraImportDialog));
   mActionExportLppz.reset(cmd.exportLppz.createAction(
       this, this, [this]() { mProjectEditor.execLppzExportDialog(this); }));
   mActionExportImage.reset(cmd.exportImage.createAction(this, this, [this]() {
@@ -595,6 +613,8 @@ void BoardEditor::createActions() noexcept {
   }));
   mActionExportStep.reset(cmd.exportStep.createAction(
       this, this, &BoardEditor::execStepExportDialog));
+  mActionExportSpecctra.reset(cmd.exportSpecctraDsn.createAction(
+      this, this, &BoardEditor::execSpecctraExportDialog));
   mActionPrint.reset(cmd.print.createAction(this, this, [this]() {
     execGraphicsExportDialog(GraphicsExportDialog::Output::Print, "print");
   }));
@@ -1017,12 +1037,14 @@ void BoardEditor::createMenus() noexcept {
   {
     MenuBuilder smb(mb.addSubMenu(&MenuBuilder::createImportMenu));
     smb.addAction(mActionImportDxf);
+    smb.addAction(mActionImportSpecctra);
   }
   {
     MenuBuilder smb(mb.addSubMenu(&MenuBuilder::createExportMenu));
     smb.addAction(mActionExportPdf);
     smb.addAction(mActionExportImage);
     smb.addAction(mActionExportStep);
+    smb.addAction(mActionExportSpecctra);
     smb.addAction(mActionExportLppz);
   }
   {
@@ -1329,21 +1351,22 @@ void BoardEditor::runDrc(bool quick) noexcept {
     // Run the DRC.
     QElapsedTimer timer;
     timer.start();
-    BoardDesignRuleCheck drc(*board, board->getDrcSettings());
+    BoardDesignRuleCheck drc;
     connect(&drc, &BoardDesignRuleCheck::progressPercent, mDockDrc.data(),
             &RuleCheckDock::setProgressPercent);
     connect(&drc, &BoardDesignRuleCheck::progressStatus, mDockDrc.data(),
             &RuleCheckDock::setProgressStatus);
-    drc.execute(quick);  // can throw
+    drc.start(*board, board->getDrcSettings(), quick);  // can throw
+    const BoardDesignRuleCheck::Result result = drc.waitForFinished();
 
     // Update DRC messages.
     clearDrcMarker();
-    mDrcMessages.insert(board->getUuid(), drc.getMessages());
-    mDockDrc->setMessages(drc.getMessages());
+    mDrcMessages.insert(board->getUuid(), result.messages);
+    mDockDrc->setMessages(result.messages);
 
     // Detect & remove disappeared messages.
     const QSet<SExpression> approvals =
-        RuleCheckMessage::getAllApprovals(drc.getMessages());
+        RuleCheckMessage::getAllApprovals(result.messages);
     if (board->updateDrcMessageApprovals(approvals, quick)) {
       mDockDrc->setApprovals(board->getDrcMessageApprovals());
       mProjectEditor.setManualModificationsMade();
@@ -1502,7 +1525,7 @@ void BoardEditor::startPlaneRebuild(bool full) noexcept {
   if (board && mPlaneFragmentsBuilder) {
     if (full) {
       // Forced rebuild -> all layers.
-      mPlaneFragmentsBuilder->startAsynchronously(*board);
+      mPlaneFragmentsBuilder->start(*board);
     } else {
       // Automatic rebuild -> only modified & visible layers. However, if the
       // 3D view is open, all planes on outer layers are visible!
@@ -1515,7 +1538,7 @@ void BoardEditor::startPlaneRebuild(bool full) noexcept {
           }
         }
       }
-      mPlaneFragmentsBuilder->startAsynchronously(*board, &layers);
+      mPlaneFragmentsBuilder->start(*board, &layers);
     }
   }
 }
@@ -1770,6 +1793,134 @@ void BoardEditor::execD356NetlistExportDialog() noexcept {
   }
 }
 
+void BoardEditor::execSpecctraExportDialog() noexcept {
+  Board* board = getActiveBoard();
+  if (!board) return;
+
+  try {
+    // Default file path.
+    QString path = "output/{{VERSION}}/{{PROJECT}}";
+    if (mProject.getBoards().count() > 1) {
+      path += "_{{BOARD}}";
+    }
+    path += ".dsn";
+    path = AttributeSubstitutor::substitute(
+        path, ProjectAttributeLookup(*board, nullptr), [&](const QString& str) {
+          return FilePath::cleanFileName(
+              str, FilePath::ReplaceSpaces | FilePath::KeepCase);
+        });
+
+    // Use memorized file path, if board path and version number match.
+    QSettings cs;
+    const QString csId =
+        board->getDirectory().getAbsPath().toStr() + *mProject.getVersion();
+    const QString csKey = "board_editor/dsn_export/" %
+        QString(QCryptographicHash::hash(csId.toUtf8(), QCryptographicHash::Md5)
+                    .toHex());
+    path = cs.value(csKey, path).toString();
+
+    // Make file path absolute.
+    if (QFileInfo(path).isRelative()) {
+      path = mProject.getPath().getPathTo(path).toStr();
+    }
+
+    // Choose file path.
+    path = FileDialog::getSaveFileName(
+        this, EditorCommandSet::instance().exportSpecctraDsn.getDisplayText(),
+        path, "*.dsn");
+    if (path.isEmpty()) return;
+    if (!path.contains(".")) path.append(".dsn");
+    const FilePath fp(path);
+
+    // Memorize file path.
+    cs.setValue(csKey,
+                fp.isLocatedInDir(mProject.getPath())
+                    ? fp.toRelative(mProject.getPath())
+                    : fp.toNative());
+
+    // Perform export.
+    qDebug().nospace() << "Export Specctra DSN to " << fp.toNative() << "...";
+    BoardSpecctraExport exp(*board);
+    FileUtils::writeFile(fp, exp.generate());  // can throw
+    qDebug() << "Successfully exported Specctra DSN.";
+    mUi->statusbar->showMessage(tr("Success!"), 3000);
+  } catch (const Exception& e) {
+    QMessageBox::critical(this, tr("Error"), e.getMsg());
+  }
+}
+
+void BoardEditor::execSpecctraImportDialog() noexcept {
+  Board* board = getActiveBoard();
+  if (!board) return;
+
+  auto logger = std::make_shared<MessageLogger>();
+  logger->warning(
+      tr("This is a new feature and we could test it only with very few "
+         "external routers. If you experience any compatibility issue with "
+         "your router, please let us know!"));
+  logger->warning(" → https://librepcb.org/help/");
+
+  try {
+    // Use memorized export file path, if board path and version number match.
+    QSettings cs;
+    const QString csId =
+        board->getDirectory().getAbsPath().toStr() + *mProject.getVersion();
+    const QString csKey = "board_editor/dsn_export/" %
+        QString(QCryptographicHash::hash(csId.toUtf8(), QCryptographicHash::Md5)
+                    .toHex());
+    QString path = cs.value(csKey).toString().replace(".dsn", ".ses");
+
+    // Make file path absolute.
+    if (QFileInfo(path).isRelative()) {
+      path = mProject.getPath().getPathTo(path).toStr();
+    }
+
+    // Choose file path.
+    path = FileDialog::getOpenFileName(
+        this, EditorCommandSet::instance().importSpecctraSes.getDisplayText(),
+        path, "*.ses;;*");
+    if (path.isEmpty()) return;
+    const FilePath fp(path);
+
+    // Set UI into busy state during the import.
+    setCursor(Qt::WaitCursor);
+    auto busyScopeGuard = scopeGuard([this]() { unsetCursor(); });
+
+    // Perform import.
+    qDebug().nospace() << "Import Specctra SES from " << fp.toNative() << "...";
+    logger->debug(tr("Parsing Specctra session '%1'...").arg(fp.toNative()));
+    const QByteArray content = FileUtils::readFile(fp);  // can throw
+    std::unique_ptr<SExpression> root =
+        SExpression::parse(content, fp, SExpression::Mode::Permissive);
+    mProjectEditor.getUndoStack().execCmd(
+        new CmdBoardSpecctraImport(*board, *root, logger));  // can throw
+    qDebug() << "Successfully imported Specctra SES.";
+  } catch (const Exception& e) {
+    logger->critical(e.getMsg());
+    logger->critical(tr("Import failed, no changes made to the board."));
+  }
+
+  // Display messages.
+  QDialog dlg(this);
+  dlg.setWindowTitle(tr("Specctra SES Import"));
+  dlg.setMinimumSize(600, 400);
+  QVBoxLayout* layout = new QVBoxLayout(&dlg);
+  QTextBrowser* txtBrowser = new QTextBrowser(&dlg);
+  txtBrowser->setReadOnly(true);
+  txtBrowser->setWordWrapMode(QTextOption::WordWrap);
+  txtBrowser->setText(
+      logger->getMessagesRichText(EditorToolbox::isWindowBackgroundDark()
+                                      ? MessageLogger::ColorTheme::Dark
+                                      : MessageLogger::ColorTheme::Light));
+  txtBrowser->verticalScrollBar()->setValue(
+      txtBrowser->verticalScrollBar()->maximum());
+  layout->addWidget(txtBrowser);
+  QPushButton* btnClose = new QPushButton(tr("Close"), &dlg);
+  connect(btnClose, &QPushButton::clicked, &dlg, &QDialog::accept);
+  layout->addWidget(btnClose);
+  dlg.exec();
+}
+
 bool BoardEditor::show3DView() noexcept {
   if (!mOpenGlView) {
     mOpenGlView.reset(new OpenGlView(this));
